@@ -402,11 +402,13 @@ terminal_window_dropdown_set_property (GObject      *object,
           icon_name = gtk_window_get_icon_name (GTK_WINDOW (object));
           if (dropdown->status_icon == NULL && icon_name != NULL)
             {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
               dropdown->status_icon = g_path_is_absolute (icon_name)
                 ? gtk_status_icon_new_from_file (icon_name)
                 : gtk_status_icon_new_from_icon_name (icon_name);
               gtk_status_icon_set_title (dropdown->status_icon, _("Drop-down Terminal"));
               gtk_status_icon_set_tooltip_text (dropdown->status_icon, _("Toggle Drop-down Terminal"));
+G_GNUC_END_IGNORE_DEPRECATIONS
               g_signal_connect (G_OBJECT (dropdown->status_icon), "button-press-event",
                   G_CALLBACK (terminal_window_dropdown_status_icon_press_event), dropdown);
               g_signal_connect (G_OBJECT (dropdown->status_icon), "popup-menu",
@@ -577,12 +579,26 @@ static gboolean
 terminal_window_dropdown_can_grab (gpointer data)
 {
   GdkWindow    *window = gtk_widget_get_window (GTK_WIDGET (data));
-  GdkGrabStatus status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
+  GdkGrabStatus status;
+#if GTK_CHECK_VERSION (3, 20, 0)
+  GdkDisplay   *display = gdk_window_get_display (window);
+  GdkSeat      *seat = gdk_display_get_default_seat (display);
+
+  status = gdk_seat_grab (seat, window,
+                          GDK_SEAT_CAPABILITY_POINTER | GDK_SEAT_CAPABILITY_KEYBOARD,
+                          FALSE, NULL, NULL, NULL, NULL);
+#else
+  status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
+#endif
 
   if (status == GDK_GRAB_SUCCESS)
     {
       /* drop the grab */
+#if GTK_CHECK_VERSION (3, 20, 0)
+      gdk_seat_ungrab (seat);
+#else
       gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+#endif
       return FALSE;
     }
 
